require 'studio_game/game'
require 'studio_game/treasure_trove'

module StudioGame
  describe Game do
    before do
      $stdout = StringIO.new
      @game = Game.new("sonic")
      @initial_health = 100
      @player = Player.new("alfian", @initial_health)
      @game.add_player(@player)
      @rounds_number = 3
    end

    it "compute total points as the sum of all player points" do
      game = Game.new("alfian")
      player1 = Player.new("johan", 42)
      player2 = Player.new("andy", 50)

      game.add_player(player1)
      game.add_player(player2)

      player1.found_treasure(Treasure.new(:excalibur, 200))
      player2.found_treasure(Treasure.new(:ragnarok, 100))

      expect(game.total_points).to eql(300)

    end


    it "assign a treasure everytime a player get a turn" do
      player = Player.new("alfian")
      game = Game.new("knuckles")
      game.add_player(player)
      game.play(1)
      expect(player.points).not_to be_zero
    end

    it "should w00t the player if high number is rolled" do
      allow_any_instance_of(Die).to receive(:roll).and_return(5)
      @game.play(@rounds_number)
      expect(@player.health).to eq(@initial_health + 15 * @rounds_number)
    end

    it "skips the player if a medium number is rolled" do
      allow_any_instance_of(Die).to receive(:roll).and_return(3)
      @game.play(@rounds_number)
      expect(@player.health).to eq(@initial_health)
    end

    it "should blam the player if low number is rolled" do
      allow_any_instance_of(Die).to receive(:roll).and_return(1)
      @game.play(@rounds_number)
      expect(@player.health).to eq(@initial_health - 10 * @rounds_number)
    end

    context "A collection of players" do
      before do 
        @player1 = Player.new("Alfian", 100)
        @player2 = Player.new("Johan", 200)
        @player3 = Player.new("Harry", 300)
        hammer = Treasure.new(:hammer, 50)
        @player1.found_treasure(hammer)
        @player2.found_treasure(hammer)
        @player3.found_treasure(hammer)

        @players = [@player1, @player2, @player3]
      end

      it "sort the player in descending order based on the score" do
        expect(@players.sort).to eq([@player3, @player2, @player1])
      end
    end
  end
end
