require 'studio_game/player'
require 'studio_game/treasure_trove'

module StudioGame
  describe Player do

    before do
      $stdout = StringIO.new
      @initial_health = 150
      @player = Player.new("alfian", @initial_health)
    end

    it "can be created from CSV string" do
      player = Player.from_csv("Johan,150")
      expect(player.name).to eql("Johan")
      expect(player.health).to eql(150)
    end


    it "yield each found treasure and its total points" do
      @player.found_treasure(Treasure.new(:excalibur, 200))
      @player.found_treasure(Treasure.new(:excalibur, 500))
      @player.found_treasure(Treasure.new(:ragnarok, 50))
      @player.found_treasure(Treasure.new(:ragnarok, 200))
      @player.found_treasure(Treasure.new(:einhander, 40))
      @player.found_treasure(Treasure.new(:einhander, 60))

      yielded = []

      @player.each_found_treasure do |treasure|
        yielded << treasure
      end

      expect(yielded).to eq([
        Treasure.new(:excalibur, 700),
        Treasure.new(:ragnarok, 250),
        Treasure.new(:einhander, 100)
        ])
    end
    
    it "computes points as sum of all treasure point" do
      @player.found_treasure(Treasure.new(:excalibur, 500))
      expect(@player.points).to eq(500)
      @player.found_treasure(Treasure.new(:einhander, 1000))
      expect(@player.points).to eq(1500)
      @player.found_treasure(Treasure.new(:ragnarok, 2000))
      expect(@player.points).to eq(3500)
    end

    it "computes the score as the sum of player's health and player's points" do
      @player.found_treasure(Treasure.new(:excalibur, 1000))
      @player.found_treasure(Treasure.new(:ragnarok, 2000))
      expect(@player.score).to eq(3150)
    end

    it "Should have a capitalized name" do
      expect(@player.name).to eql("Alfian")
    end

    it "Has an initial health" do 
      expect(@player.health).to eql(@initial_health)
    end

    it "has a string representation" do
      @player.found_treasure(Treasure.new(:excalibur, 1000))

      expect(@player.to_s).to eq("I am #{@player.name} with health: #{@player.health}, points: #{@player.points} and score: #{@player.score}")
    end

    it "Computes a score as the sum of health and player points" do
      @player.found_treasure(Treasure.new(:excalibur, 1000))
      expect(@player.score).to eq(@player.health + @player.points)
    end

    it "Increases health by 15 when w00ted" do 
      @player.w00t
      expect(@player.health).to eq(@initial_health + 15)
    end

    it "Decreases health by 10 when blammed" do
      @player.blam
      expect(@player.health).to eq(@initial_health - 10)
    end

    context "Create a player with strong parameter" do
      before do
        @player = Player.new("alfian", 150)
      end

      it "is a strong player" do
        expect(@player).to be_strong
      end
    end

    context "Create a player with wimpy parameter" do
      before do
        @player = Player.new("alfian", 100)
      end

      it "is not a strong player" do
        expect(@player).not_to be_strong
      end
    end

  end
end